@@ -15,13 +15,16 @@ use boolean qw(true false);
 
 use Carp qw(croak);
 use DateTime ();
+use DateTime::HiRes ();
 use DateTime::TimeZone ();
 use List::MoreUtils qw(all any none);
 use Params::Validate ':all';
 use Scalar::Util qw(blessed);
 use Storable qw(dclone);
 
-our $VERSION = '1.06';
+use DateTime::Format::Natural::Utils qw(trim);
+
+our $VERSION = '1.16';
 
 validation_options(
     on_fail => sub
@@ -53,6 +56,7 @@ sub _init
     my %presets = (
         lang          => 'en',
         format        => 'd/m/y',
+        demand_future =>  false,
         prefer_future =>  false,
         time_zone     => 'floating',
     );
@@ -67,10 +71,12 @@ sub _init
     $self->{Daytime} = $opts{daytime} || {};
 
     my $mod = join '::', (__PACKAGE__, 'Lang', uc $self->{Lang});
-    eval "require $mod"; die $@ if $@;
+    eval "require $mod" or die $@;
 
     $self->{data} = $mod->__new();
     $self->{grammar_class} = $mod;
+
+    $self->{mode} = '';
 }
 
 sub _init_check
@@ -78,6 +84,18 @@ sub _init_check
     my $self = shift;
 
     validate(@_, {
+        demand_future => {
+            # SCALARREF due to boolean.pm's implementation
+            type => BOOLEAN | SCALARREF,
+            optional => true,
+            callbacks => {
+                'mutually exclusive' => sub
+                {
+                    return true unless exists $_[1]->{prefer_future};
+                    die "prefer_future provided\n";
+                },
+            },
+        },
         lang => {
             type => SCALAR,
             optional => true,
@@ -86,12 +104,23 @@ sub _init_check
         format => {
             type => SCALAR,
             optional => true,
-            regex => qr!^(?:[dmy]{1,4}[-./]){2}[dmy]{1,4}$!i,
+            regex => qr!^(?:
+                           (?: (?: [dmy]{1,4}[-./] ){2}[dmy]{1,4} )
+                             |
+                           (?: [dm]{1,2}/[dm]{1,2} )
+                         )$!ix,
         },
         prefer_future => {
             # SCALARREF due to boolean.pm's implementation
             type => BOOLEAN | SCALARREF,
             optional => true,
+            callbacks => {
+                'mutually exclusive' => sub
+                {
+                    return true unless exists $_[1]->{demand_future};
+                    die "demand_future provided\n";
+                },
+            },
         },
         time_zone => {
             type => SCALAR | OBJECT,
@@ -143,6 +172,8 @@ sub parse_datetime
 
     $self->{input_string} = $self->{date_string};
 
+    $self->{mode} = 'parse';
+
     my $date_string = $self->{date_string};
 
     $self->_rewrite(\$date_string);
@@ -161,7 +192,7 @@ sub parse_datetime
         my $dt = $self->_parse_formatted_md($date_string);
         return $dt if blessed($dt);
 
-        if ($self->{Prefer_future}) {
+        if ($self->{Prefer_future} || $self->{Demand_future}) {
             $self->_advance_future('md');
         }
     }
@@ -188,6 +219,8 @@ sub parse_datetime
 
         $self->_set(%args);
 
+        $self->{datetime}->truncate(to => 'second');
+        $self->_set_truncated;
         $self->_set_valid_exp;
     }
     elsif ($date_string =~ /^([+-]) (\d+?) ([a-zA-Z]+)$/x) {
@@ -224,6 +257,8 @@ sub parse_datetime
 
         $self->_set(%args);
 
+        $self->{datetime}->truncate(to => 'second');
+        $self->_set_truncated;
         $self->_set_valid_exp;
     }
     else {
@@ -259,12 +294,7 @@ sub _params_init
         (${$params->{string}}) = @_;
     }
 
-    ${$params->{string}} = do {
-        local $_ = ${$params->{string}};
-        s/^\s+//;
-        s/\s+$//;
-        $_
-    };
+    trim($params->{string});
 }
 
 sub _parse_init
@@ -281,7 +311,7 @@ sub _parse_init
             $self->{datetime} = dclone($self->{Datetime});
         }
         else {
-            $self->{datetime} = DateTime->$method(
+            $self->{datetime} = DateTime::HiRes->$method(
                 time_zone => $self->{Time_zone},
                 %$args,
             );
@@ -301,6 +331,7 @@ sub _parse_init
     $self->_unset_error;
     $self->_unset_valid_exp;
     $self->_unset_trace;
+    $self->_unset_truncated;
 }
 
 sub parse_datetime_duration
@@ -327,9 +358,9 @@ sub parse_datetime_duration
     }
 
     $self->_pre_duration(\@date_strings);
-    $self->{state} = {};
+    @$self{qw(state truncated_duration)} = ({}, []);
 
-    my (@queue, @traces);
+    my (@queue, @traces, @truncated);
     foreach my $date_string (@date_strings) {
         push @queue, $self->parse_datetime($date_string);
         $self->_save_state(
@@ -340,15 +371,19 @@ sub parse_datetime_duration
         if (@{$self->{traces}}) {
             push @traces, $self->{traces}[0];
         }
+        if ($self->{running_tests}) {
+            push @truncated, $self->_get_truncated;
+        }
     }
 
-    $self->_post_duration(\@queue, \@traces);
+    $self->_post_duration(\@queue, \@traces, \@truncated);
     $self->_restore_state;
 
     delete @$self{qw(duration insert state)};
 
-    @{$self->{traces}} = @traces;
-    $self->{input_string} = $duration_string;
+    @{$self->{traces}}             = @traces;
+    @{$self->{truncated_duration}} = @truncated;
+    $self->{input_string}          = $duration_string;
 
     if ($shrinked) {
         $self->_set_failure;
@@ -365,8 +400,18 @@ sub extract_datetime
     my $extract_string;
     $self->_params_init(@_, { string => \$extract_string });
 
+    $self->_unset_failure;
+    $self->_unset_error;
+    $self->_unset_valid_exp;
+
+    $self->{input_string} = $extract_string;
+
+    $self->{mode} = 'extract';
+
     my @expressions = $self->_extract_expressions($extract_string);
 
+    $self->_set_valid_exp if @expressions;
+
     return wantarray ? @expressions : $expressions[0];
 }
 
@@ -383,8 +428,22 @@ sub error
 
     return '' if $self->success;
 
-    my $error  = "'$self->{input_string}' does not parse ";
-       $error .= $self->_get_error || '(perhaps you have some garbage?)';
+    my $error = sub
+    {
+        return undef unless defined $self->{mode} && length $self->{mode};
+        my %errors = (
+            extract => "'$self->{input_string}' cannot be extracted from",
+            parse   => "'$self->{input_string}' does not parse",
+        );
+        return $errors{$self->{mode}};
+    }->();
+
+    if (defined $error) {
+        $error .= ' ' . ($self->_get_error || '(perhaps you have some garbage?)');
+    }
+    else {
+        $error = 'neither extracting nor parsing method invoked';
+    }
 
     return $error;
 }
@@ -393,7 +452,7 @@ sub trace
 {
     my $self = shift;
 
-    return @{$self->{traces}};
+    return @{$self->{traces} || []};
 }
 
 sub _process
@@ -511,6 +570,7 @@ sub _truncate
         my $index = $indexes{$unit} - 1;
         if (defined $units[$index] && !exists $self->{modified}{$units[$index]}) {
             $self->{datetime}->truncate(to => $unit);
+            $self->_set_truncated;
             last;
         }
     }
@@ -523,8 +583,8 @@ sub _post_process
 
     delete $opts{truncate_to};
 
-    if ($self->{Prefer_future} &&
-        (exists $opts{prefer_future} && $opts{prefer_future})
+    if (($self->{Prefer_future} || $self->{Demand_future})
+        && (exists $opts{advance_future} && $opts{advance_future})
     ) {
         $self->_advance_future;
     }
@@ -550,17 +610,39 @@ sub _advance_future
 
     my $now = exists $self->{Datetime}
       ? dclone($self->{Datetime})
-      : DateTime->now(time_zone => $self->{Time_zone});
+      : DateTime::HiRes->now(time_zone => $self->{Time_zone});
 
-    if ((all { /^(?:second|minute|hour)$/ } keys %modified)
+    my $day_of_week = sub { $_[0]->_Day_of_Week(map $_[0]->{datetime}->$_, qw(year month day)) };
+
+    my $skip_weekdays = false;
+
+    if ((all { /^(?:(?:nano)?second|minute|hour)$/ } keys %modified)
         && (exists $self->{modified}{hour} && $self->{modified}{hour} == 1)
-        && $self->{datetime}->hour < $now->hour
+        && (($self->{Prefer_future} && $self->{datetime} <  $now)
+         || ($self->{Demand_future} && $self->{datetime} <= $now))
     ) {
         $self->{postprocess}{day} = 1;
     }
-    elsif ($token_contains->('weekdays_all')
+    elsif (sub {
+        return false unless @{$self->{tokens}} == 2;
+        my ($day, $weekday) = map $self->{data}->__RE($_), qw(day weekday);
+        if ($self->{tokens}->[0] =~ $day
+         && $self->{tokens}->[1] =~ $weekday) {
+            $skip_weekdays = true;
+            return true;
+        }
+        return false;
+    }->()
+        && (all { /^(?:day|month|year)$/ } keys %modified)
+        && (($self->{Prefer_future} && $self->{datetime}->day <  $now->day)
+         || ($self->{Demand_future} && $self->{datetime}->day <= $now->day))
+    ) {
+        $self->{postprocess}{week} = 4;
+    }
+    elsif (($token_contains->('weekdays_all') && !$skip_weekdays)
         && (exists $self->{modified}{day} && $self->{modified}{day} == 1)
-        && ($self->_Day_of_Week(map $self->{datetime}->$_, qw(year month day)) < $now->wday)
+        && (($self->{Prefer_future} && $day_of_week->($self) <  $now->wday)
+         || ($self->{Demand_future} && $day_of_week->($self) <= $now->wday))
     ) {
         $self->{postprocess}{day} = 7;
     }
@@ -571,7 +653,8 @@ sub _advance_future
               ? $self->{modified}{day} == 1
                 ? true : false
               : true)
-        && ($self->{datetime}->day_of_year < $now->day_of_year)
+        && (($self->{Prefer_future} && $self->{datetime}->day_of_year <  $now->day_of_year)
+         || ($self->{Demand_future} && $self->{datetime}->day_of_year <= $now->day_of_year))
     ) {
         $self->{postprocess}{year} = 1;
     }
@@ -605,18 +688,23 @@ sub _get_valid_exp   { $_[0]->{valid_expression}         }
 sub _set_valid_exp   { $_[0]->{valid_expression} = true  }
 sub _unset_valid_exp { $_[0]->{valid_expression} = false }
 
+sub _get_truncated   { $_[0]->{truncated}         }
+sub _set_truncated   { $_[0]->{truncated} = true  }
+sub _unset_truncated { $_[0]->{truncated} = false }
+
 sub _get_datetime_object
 {
     my $self = shift;
 
     my $dt = DateTime->new(
-        time_zone => $self->{datetime}->time_zone,
-        year      => $self->{datetime}->year,
-        month     => $self->{datetime}->month,
-        day       => $self->{datetime}->day_of_month,
-        hour      => $self->{datetime}->hour,
-        minute    => $self->{datetime}->minute,
-        second    => $self->{datetime}->second,
+        time_zone  => $self->{datetime}->time_zone,
+        year       => $self->{datetime}->year,
+        month      => $self->{datetime}->month,
+        day        => $self->{datetime}->day_of_month,
+        hour       => $self->{datetime}->hour,
+        minute     => $self->{datetime}->minute,
+        second     => $self->{datetime}->second,
+        nanosecond => $self->{datetime}->nanosecond,
     );
 
     foreach my $unit (keys %{$self->{postprocess}}) {
@@ -703,7 +791,8 @@ not necessarily required.
            datetime      => DateTime->new(...),
            lang          => 'en',
            format        => 'mm/dd/yy',
-           prefer_future => '[0|1]',
+           prefer_future => [0|1],
+           demand_future => [0|1],
            time_zone     => 'floating',
            daytime       => { morning   => 06,
                               afternoon => 13,
@@ -724,12 +813,37 @@ Defaults to 'C<en>'.
 
 =item * C<format>
 
-Specifies the format of numeric dates, defaults to 'C<d/m/y>'.
+Specifies the format of numeric dates.
+
+The format is used to influence how numeric dates are parsed. Given two
+numbers separated by a slash, the month/day order expected comes from
+this option. If there is a third number, this option describes where
+to expect the year. When this format can't be used to interpret the
+date, some unambiguous dates may be parsed, but there is no form
+guarantee.
+
+Current supported "month/day" formats: C<dd/mm>, C<mm/dd>.
+
+Current supported "year/month/day" formats (with slashes): C<dd/mm/yy>,
+C<dd/mm/yyyy>, C<mm/dd/yyyy>, C<yyyy/mm/dd>.
+
+Note that all of the above formats with three units do also parse
+with dots or dashes as format separators.
+
+Furthermore, formats can be abbreviated as long as they remain
+unambiguous.
+
+Defaults to 'C<d/m/y>'.
 
 =item * C<prefer_future>
 
 Prefers future time and dates. Accepts a boolean, defaults to false.
 
+=item * C<demand_future>
+
+Demands future time and dates. Similar to C<prefer_future>, but stronger.
+Accepts a boolean, defaults to false.
+
 =item * C<time_zone>
 
 The time zone to use when parsing and for output. Accepts any time zone
@@ -869,6 +983,11 @@ valuable suggestions and patches:
  Debian Perl Group
  Tim Bunce
  Ricardo Signes
+ Felix Ostmann
+ Jrn Clausen
+ Jim Avera
+ Olaf Alders
+ Karen Etheridge
 
 =head1 SEE ALSO
 
